library ormicida.constructors;

import 'dart:mirrors';

class DefaultConstructor {
  final Type                  type;
  final Symbol                name;
  final List                  positional;
  final Map<Symbol, dynamic>  named;
  
  const DefaultConstructor(this.type, {
    this.name: const Symbol(''),
    this.positional: const [],
    this.named: null
  });
  
  InstanceMirror newInstanceMirror() =>
      reflectClass(type).newInstance(name, positional, named);
  
  dynamic call(_) => newInstanceMirror().reflectee;
}

class CustomConstructor {
  final Type                type;
  final Symbol              name;
  final List<String>        positional;
  final Map<Symbol, String> named;
  
  const CustomConstructor(this.type, {
    this.name: const Symbol(''),
    this.positional: const <String>[],
    this.named: null
  });
  
  List<dynamic> _buildPositional(Map<String, dynamic> arguments) {
    if (positional == null) {
      return const [];
    } else {
      return new List.from(
          positional.map(
              (String name) => arguments[name]),
              growable: false);
    }
  }
  
  Map<Symbol, dynamic> _buildNamed(Map<String, dynamic> arguments) {
    if (named == null) {
      return null;
    } else {
      final result = {};
      named.forEach((Symbol key, String value) {
        result[key] = arguments[value];
      });
      return result;
    }
  }
  
  InstanceMirror newInstanceMirror(Map<String, dynamic> arguments) =>
      reflectClass(type).newInstance(name,
          _buildPositional(arguments), _buildNamed(arguments));
  
  dynamic call(Map<String, dynamic> arguments) =>
      newInstanceMirror(arguments).reflectee;
}
